%This file contains examples of code using the approximate roots with
%multiplicities code

%Expressing multivariate polynomials
%
%Multivariate polynomials are expressed by a coefficient, support pair
%The coefficient vector is a row vector containing the coefficients 
%of the polynomial.
%The support matrix is a matrix containing the exponents of each monomial
%in the polynomial.  The lexographic ordering is used for monomial
%exponents.  If the monomials are out of order, mpolysimplify will reorder
%them
%
%For example:  1 + 2x + 3xy would be represented by the pair [1, 2, 3],
% [0 0;1 0;1 1]

p1 = [1, 2, 3]
s1 = [0 0;1 0;1 1]

%mpolyadd and mpolymult can be used to manipulate polynomials

p2 = 4
s2 = [0 1]

[padd, sadd] = mpolyadd(p1, s1, p2, s2)
[pmult, smult] = mpolymult(p1, s1, p2, s2) 

%mpolyeval(p, s, x) is used to evaluate the polynomial (p, s) at the point
%x

val = mpolyval(p1, s1, [1 2])

%differentiation is done with mpolydiff(p, s, diffp, diffs)
%to differentiate (p1, s1) by x you could

[p1x, s1x] = mpolydiff(p1, s1, [1], [1])

%to apply the operator (dx + 2*dy) to (p2, s2) you can

[p2d, s2d] = mpolydiff(p2, s2, [1 2], [1 0;0 1])

%to create a generalized Vandermonde matrix you need three things
% 1) The points that each monomial should be evalueated at
% 2) The support polynomials
% 3) The tangential conditions to be satisfied at each point
%
%  The point coordinates are stored as rows of a matrix
% for example the points (1, 2), (4, 5) would be stored as

pts = [1 2;4 5]
% The support polynomials are stored in a cell (i.e. in {}) as coefficient,
% support pairs.  You also need to keep track of the number of support
% polynomials
% For example, if the support polynomials are (1, x, x+y) would be stored

support = {[1], [0], [1], [1], [1, 1], [1, 0;0, 1]}
nsup = 3

% The tangential conditions are store in a similar way, but you need a
% seperate cell for each point.
% You also need to keep track of the number of tangential conditions for
% each point.
% So if the the first point should be a simple root
% and the second has dual basis (1, x) you use

tconds = {{[1], [0]}, {[1], [0], [1], [1]}}
nconds = [1, 2]

% The call to make the Vandermonde matrix is

V = VandMat(pts, nsup, nconds, support, tconds)

%To run the standard algorithm, you need 5 things
%
%1) The polynomial system (as a cell of coeffcient, exponent pairs)
%2) The support polynomials (as a cell of coeff, exp pairs)
%3) The tangential conditions to be satisfied (as a cell of cells)
%4) The initial iterate
%5) The maximum allowed iterations
%
%It returns
%1) The perturbed system
%2) The calculated common roots
%3) The number of iterations completed
%4) The residual (2-norm of the evaluation vector of the perturbed system
%at the calculated roots)
%
%For example, the system 1+3x^2+5y^2, 2 + 4xy would be

sys = {[1 3 5] [0 0;2 0;0 2] [2 4] [0 0;2 4]}

%A support of {1, x, y, 2x+y} would be

sup = {[1] [0 0] [1] [1 0] [1] [0 1] [2 1] [2 0;0 1]}

%If you are looking for two roots
%one simple, the second having dual basis {1, dx, dxdy} it would be

tconds = { {[1] [0]}, {[1] [0] [1] [1] [1] [1 1]} }

%an initial iterate of (1, 1) (2, 3) would be

init = [1 1;2 3]

%and the call would be

[F, rts, iter, res] = ClosestPoly(sys,sup,tconds, init, 150);


%To do the simplified version of the iteration, the call is

[F, rts, iter, res] = ClosestPolySimp(sys,sup,tconds, init, 150);

%The arbitrary support iteration is automatically called when
%you give more support than constraints to ClosestPoly
%You can also call it directly with

[F, rts, iter, res] = ClosestPolyArbSup(sys,sup,tconds, init, 150);

%the file testbatch.m has command lines for running automated tests
%see runtest.m for an explanation of command line arguments
%we recreate test cases by specifing a particular seed to the random
%number generator before creating random polynomials.  See runtestcase.m
%for details.
%